'use strict';

/**
 * @name taskify
 *
 * @description
 * Lorem ipsum dolor sit amet
 *
 * @requires fs
 * @requires path
 * @requires gulp
 * @requires ./createTask
 *
 * @param {object} options
 * Lorem ipsum dolor sit amet
 *
 * @returns {array} tasks
 * Lorem ipsum dolor sit amet
 */
module.exports = function taskify (options) {

    var
        //
        fs   = require('fs'),
        path = require('path'),

        //
        gulp = require('gulp'),

        //
        createTask = require('./createTask.js'),

        //
        defaults = {
            'seperator' : '--',
            'subtasks'  : 'subtasks/',
            'tasksDir'  : 'build/tasks/'
        },

        // Chop the name off
        args = Array.prototype.slice.call(arguments, 1),

        //
        origDeps = [],

        //
        tasks;

    //
    if (typeof options === 'string') {
        options = {
            'seperator' : defaults.seperator,
            'subtasks'  : defaults.subtasks,
            'tasksDir'  : options
        };

    //
    } else if (options == null) {
        options = defaults;

    //
    } else {

        //
        if (!('seperator' in options)) {
            options.seperator = defaults.seperator;
        }

        //
        if (!('subtasks' in options)) {
            options.subtasks = defaults.subtasks;
        }

        //
        if (!('tasksDir' in options)) {
            options.tasksDir = defaults.tasksDir;
        }

    }

    //
    if (!options.tasksDir || typeof options.tasksDir !== 'string') {
        throw new Error();
    }

    //
    try {
        options.tasksDir = path.normalize(options.tasksDir);
    } catch (err) {
        throw err;
    }

    // Makes the path relative to the root folder, eg: .git
    try {
        options.tasksDir = path.join(process.cwd(), options.tasksDir);
    } catch (err) {
        throw err;
    }

    //
    if (!options.subtasks || typeof options.subtasks !== 'string') {
        throw new Error();
    }

    //
    if (typeof options.seperator === 'string') {

        //
        options.seperator = (function (sep) {
            return function (taskName, subtaskName) {
                return taskName + sep + subtaskName;
            };
        })(options.seperator);

    //
    } else if (typeof options.seperator !== 'function') {
        throw new Error();
    }

    //
    try {
        tasks = fs.lstatSync(options.tasksDir);
    } catch (err) {
        throw err;
    }

    //
    if (tasks.isDirectory()) {

        //
        try {
            tasks = fs.readdirSync(options.tasksDir);
        } catch (err) {
            throw err;
        }

        //
        if (!tasks.length) {
            throw new Error();
        }

    //
    } else {
        throw new Error();
    }

    //
    args.unshift(gulp);

    //
    tasks = tasks.map(function (dirName) {

        var
            //
            taskObj,

            //
            taskPath;

        //
        try {
            taskPath = path.join(options.tasksDir, dirName);
        } catch (err) {
            throw err;
        }

        //
        try {
            taskObj = createTask(options, taskPath, args);
        } catch (err) {
            throw err;
        }

        // Shallow copy
        origDeps = taskObj.deps.slice();

        //
        taskObj.subtasks.forEach(function (subtaskObj) {

            //
            if (subtaskObj.auto) {

                // Don't pollute
                origDeps.forEach(function (dep) {

                    //
                    if (subtaskObj.deps.indexOf(dep) === -1) {
                        subtaskObj.deps.push(dep);
                    }

                });

                //
                taskObj.deps.push(subtaskObj.name);

            }

            //
            subtaskObj.alias.forEach(function (aliasName) {
                gulp.task(aliasName, subtaskObj.deps, subtaskObj.exports);
            });

            //
            gulp.task(subtaskObj.name, subtaskObj.deps, subtaskObj.exports);

        });

        //
        taskObj.alias.forEach(function (aliasName) {
            gulp.task(aliasName, taskObj.deps, taskObj.exports);
        });

        //
        gulp.task(taskObj.name, taskObj.deps, taskObj.exports);

        //
        return taskObj;

    });

    //
    return tasks;

};
